#!/usr/bin/env python
"""
Tests for FastJango app configuration functionality.
"""

import os
import sys
import unittest
from unittest.mock import patch, MagicMock

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango app configuration components
from fastjango.apps import AppConfig
from fastjango.apps.registry import Apps, apps


class CustomAppConfig(AppConfig):
    """Custom app configuration for testing."""
    
    name = "test_app"
    verbose_name = "Test App"
    
    def ready(self):
        """App is ready callback."""
        self.ready_called = True


class SimpleAppConfig(AppConfig):
    """Simple app configuration for testing."""
    
    name = "simple_app"


class AppConfigTest(unittest.TestCase):
    """Test suite for AppConfig classes."""
    
    def test_basic_app_config(self):
        """Test basic AppConfig functionality."""
        # Create an AppConfig instance
        app_config = AppConfig(app_name="test_app", app_module=None)
        
        # Check attributes
        self.assertEqual(app_config.name, "test_app")
        self.assertEqual(app_config.verbose_name, "Test App")  # Should auto-convert
        self.assertEqual(app_config.label, "test_app")
        
        # Check get_model returns None for invalid model
        self.assertIsNone(app_config.get_model("NonExistentModel"))
    
    def test_custom_app_config(self):
        """Test custom AppConfig subclass."""
        # Create a custom AppConfig instance
        app_config = CustomAppConfig(app_name="test_app", app_module=None)
        
        # Check attributes
        self.assertEqual(app_config.name, "test_app")
        self.assertEqual(app_config.verbose_name, "Test App")
        
        # Call ready and check that it was called
        app_config.ready()
        self.assertTrue(hasattr(app_config, "ready_called"))
        self.assertTrue(app_config.ready_called)
    
    def test_app_config_label(self):
        """Test deriving app label from name."""
        # Full module path
        app_config1 = AppConfig(app_name="project.apps.myapp", app_module=None)
        self.assertEqual(app_config1.label, "myapp")
        
        # Simple name
        app_config2 = AppConfig(app_name="myapp", app_module=None)
        self.assertEqual(app_config2.label, "myapp")
    
    def test_get_models(self):
        """Test getting models from an app."""
        # Create a mock app module with models
        class MockModel1:
            pass
        
        class MockModel2:
            pass
        
        # Create a mock module
        mock_module = MagicMock()
        mock_module.MockModel1 = MockModel1
        mock_module.MockModel2 = MockModel2
        
        # Create app config with models
        app_config = AppConfig(app_name="test_app", app_module=mock_module)
        app_config.models = {"mockmodel1": MockModel1, "mockmodel2": MockModel2}
        
        # Check get_models
        models = app_config.get_models()
        self.assertEqual(len(models), 2)
        self.assertIn(MockModel1, models)
        self.assertIn(MockModel2, models)
        
        # Check get_model
        self.assertEqual(app_config.get_model("MockModel1"), MockModel1)
        self.assertEqual(app_config.get_model("mockmodel1"), MockModel1)  # Case insensitive


class AppsRegistryTest(unittest.TestCase):
    """Test suite for Apps registry."""
    
    def setUp(self):
        """Set up test environment."""
        # Create a fresh Apps registry
        self.apps = Apps()
    
    def test_register_app_config(self):
        """Test registering an app configuration."""
        # Create and register an app config
        app_config = SimpleAppConfig(app_name="simple_app", app_module=None)
        self.apps.register_app_config(app_config)
        
        # Check that the app is registered
        self.assertIn("simple_app", self.apps.app_configs)
        self.assertEqual(self.apps.app_configs["simple_app"], app_config)
    
    def test_get_app_config(self):
        """Test getting an app configuration."""
        # Create and register an app config
        app_config = SimpleAppConfig(app_name="simple_app", app_module=None)
        self.apps.register_app_config(app_config)
        
        # Check that get_app_config returns the correct AppConfig
        self.assertEqual(self.apps.get_app_config("simple_app"), app_config)
        
        # Check that get_app_config raises LookupError for unknown app
        with self.assertRaises(LookupError):
            self.apps.get_app_config("unknown_app")
    
    def test_is_installed(self):
        """Test checking if an app is installed."""
        # Create and register an app config
        app_config = SimpleAppConfig(app_name="simple_app", app_module=None)
        self.apps.register_app_config(app_config)
        
        # Check that is_installed returns True for installed app
        self.assertTrue(self.apps.is_installed("simple_app"))
        
        # Check that is_installed returns False for unknown app
        self.assertFalse(self.apps.is_installed("unknown_app"))
    
    def test_get_model(self):
        """Test getting a model."""
        # Create a mock model
        class MockModel:
            pass
        
        # Create and register an app config with the model
        app_config = SimpleAppConfig(app_name="simple_app", app_module=None)
        app_config.models = {"mockmodel": MockModel}
        self.apps.register_app_config(app_config)
        
        # Check that get_model returns the correct model
        self.assertEqual(
            self.apps.get_model("simple_app", "MockModel"),
            MockModel
        )
        
        # Check that get_model returns None for unknown model
        self.assertIsNone(self.apps.get_model("simple_app", "UnknownModel"))
        
        # Check that get_model raises LookupError for unknown app
        with self.assertRaises(LookupError):
            self.apps.get_model("unknown_app", "MockModel")
    
    def test_populate(self):
        """Test populating apps from installed apps setting."""
        # Mock settings with INSTALLED_APPS
        with patch("fastjango.conf.settings") as mock_settings:
            mock_settings.INSTALLED_APPS = [
                "tests.test_app",
                "simple_app",
            ]
            
            # Mock importlib
            with patch("importlib.import_module") as mock_import:
                # Mock imported modules
                mock_test_app = MagicMock()
                mock_test_app.default_app_config = "tests.test_app.apps.TestAppConfig"
                
                mock_test_app_apps = MagicMock()
                mock_test_app_apps.TestAppConfig = CustomAppConfig
                
                mock_simple_app = MagicMock()
                
                # Configure import_module to return our mocks
                def mock_import_side_effect(name):
                    if name == "tests.test_app":
                        return mock_test_app
                    elif name == "tests.test_app.apps":
                        return mock_test_app_apps
                    elif name == "simple_app":
                        return mock_simple_app
                    raise ImportError(f"No module named '{name}'")
                
                mock_import.side_effect = mock_import_side_effect
                
                # Call populate
                self.apps.populate(settings=mock_settings)
                
                # Check that the apps are registered
                self.assertIn("test_app", self.apps.app_configs)
                self.assertIn("simple_app", self.apps.app_configs)
                
                # Check that the correct AppConfig classes were used
                self.assertIsInstance(self.apps.app_configs["test_app"], CustomAppConfig)
                self.assertIsInstance(self.apps.app_configs["simple_app"], AppConfig)


class SingletonAppsTest(unittest.TestCase):
    """Test suite for the singleton apps instance."""
    
    def test_singleton_apps(self):
        """Test that apps is a singleton."""
        # The global apps object should be an Apps instance
        self.assertIsInstance(apps, Apps)
        
        # Creating a new Apps instance should not replace the singleton
        new_apps = Apps()
        self.assertIsNot(new_apps, apps)


def run_tests():
    """Run the app configuration tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


#!/usr/bin/env python
"""
Tests for FastJango middleware functionality.
"""

import os
import sys
import asyncio
import unittest
from unittest.mock import MagicMock, patch

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango middleware components
from fastapi import FastAPI, Request, Response
from fastjango.middleware.cors import CORSMiddleware
from fastjango.middleware.security import SecurityMiddleware
from fastjango.http import HttpResponse


class TestMiddleware:
    """A test middleware for unit testing."""
    
    def __init__(self, app, option=None):
        self.app = app
        self.option = option
        self.calls = []
    
    async def __call__(self, scope, receive, send):
        self.calls.append(("enter", scope["path"]))
        
        # Modify request
        if scope.get("path") == "/modified":
            scope["modified_by_middleware"] = True
        
        # Call the next middleware or endpoint
        await self.app(scope, receive, send)
        
        self.calls.append(("exit", scope["path"]))


class MiddlewareTest(unittest.TestCase):
    """Test suite for middleware functionality."""
    
    def setUp(self):
        """Set up test environment."""
        self.app = FastAPI()
        
        @self.app.get("/")
        async def index():
            return {"message": "Hello World"}
        
        @self.app.get("/modified")
        async def modified(request: Request):
            is_modified = getattr(request.scope, "modified_by_middleware", False)
            return {"modified": is_modified}
    
    def test_middleware_execution(self):
        """Test that middleware is executed in the correct order."""
        test_middleware = TestMiddleware(self.app)
        app_with_middleware = TestMiddleware(test_middleware)
        
        # Create mock scope, receive, and send
        scope = {"type": "http", "path": "/"}
        receive = MagicMock()
        send = MagicMock()
        
        # Run the middleware chain
        asyncio.run(app_with_middleware(scope, receive, send))
        
        # Check the call order
        expected_calls = [
            ("enter", "/"),  # Outer middleware enters
            ("enter", "/"),  # Inner middleware enters
            ("exit", "/"),   # Inner middleware exits
            ("exit", "/")    # Outer middleware exits
        ]
        self.assertEqual(app_with_middleware.calls + test_middleware.calls, expected_calls)
    
    def test_middleware_modifies_request(self):
        """Test that middleware can modify the request."""
        # Add the test middleware to the app
        self.app.add_middleware(TestMiddleware)
        
        # Create test client
        from fastapi.testclient import TestClient
        client = TestClient(self.app)
        
        # Make request to the modified endpoint
        response = client.get("/modified")
        
        # The middleware should have set modified_by_middleware=True
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {"modified": True})


class CORSMiddlewareTest(unittest.TestCase):
    """Test suite for CORS middleware."""
    
    def setUp(self):
        """Set up test environment."""
        self.app = FastAPI()
        
        @self.app.get("/")
        async def index():
            return {"message": "Hello World"}
        
        # Add CORS middleware
        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=["https://example.com"],
            allow_credentials=True,
            allow_methods=["GET", "POST"],
            allow_headers=["X-Custom"],
        )
        
        # Create test client
        from fastapi.testclient import TestClient
        self.client = TestClient(self.app)
    
    def test_cors_headers_preflight(self):
        """Test CORS headers are added to preflight requests."""
        response = self.client.options(
            "/",
            headers={
                "Origin": "https://example.com",
                "Access-Control-Request-Method": "GET",
                "Access-Control-Request-Headers": "X-Custom",
            },
        )
        
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["access-control-allow-origin"], "https://example.com")
        self.assertEqual(response.headers["access-control-allow-credentials"], "true")
        self.assertEqual(response.headers["access-control-allow-methods"], "GET,POST")
        self.assertEqual(response.headers["access-control-allow-headers"], "X-Custom")
    
    def test_cors_headers_simple_request(self):
        """Test CORS headers are added to simple requests."""
        response = self.client.get("/", headers={"Origin": "https://example.com"})
        
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["access-control-allow-origin"], "https://example.com")
        self.assertEqual(response.headers["access-control-allow-credentials"], "true")


class SecurityMiddlewareTest(unittest.TestCase):
    """Test suite for security middleware."""
    
    def setUp(self):
        """Set up test environment."""
        self.app = FastAPI()
        
        @self.app.get("/")
        async def index():
            return {"message": "Hello World"}
        
        # Add security middleware
        self.app.add_middleware(
            SecurityMiddleware,
            enable_hsts=True,
            hsts_max_age=31536000,
            enable_xss_protection=True,
        )
        
        # Create test client
        from fastapi.testclient import TestClient
        self.client = TestClient(self.app)
    
    def test_security_headers(self):
        """Test security headers are added to responses."""
        response = self.client.get("/")
        
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers["strict-transport-security"],
            f"max-age=31536000; includeSubDomains"
        )
        self.assertEqual(response.headers["x-xss-protection"], "1; mode=block")
        self.assertEqual(response.headers["x-content-type-options"], "nosniff")
        self.assertEqual(response.headers["x-frame-options"], "DENY")


def run_tests():
    """Run the middleware tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


"""
Settings for {{project_name}} project.
"""

import os
from pathlib import Path

# Build paths inside the project
BASE_DIR = Path(__file__).resolve().parent.parent

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-fastjango-development-key-change-in-production"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition
INSTALLED_APPS = [
    # FastJango apps
    "{{project_name_snake}}.core",
    
    # Third-party apps
    
    # Your apps
]

# Middleware
MIDDLEWARE = [
    "fastjango.middleware.security.SecurityMiddleware",
    "fastjango.middleware.session.SessionMiddleware",
    "fastapi.middleware.cors.CORSMiddleware",
    "fastjango.middleware.common.CommonMiddleware",
    "fastjango.middleware.csrf.CsrfMiddleware",
    "fastjango.middleware.auth.AuthenticationMiddleware",
    "fastjango.middleware.messages.MessageMiddleware",
]

ROOT_URLCONF = "{{project_name_snake}}.urls"

TEMPLATES = [
    {
        "BACKEND": "fastjango.template.backends.jinja2.Jinja2",
        "DIRS": [BASE_DIR / "templates"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "fastjango.template.context_processors.debug",
                "fastjango.template.context_processors.request",
                "fastjango.template.context_processors.auth",
                "fastjango.template.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "{{project_name_snake}}.wsgi.application"
ASGI_APPLICATION = "{{project_name_snake}}.asgi.application"

# Database
DATABASES = {
    "default": {
        "ENGINE": "fastjango.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

# Password validation
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "fastjango.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "fastjango.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "fastjango.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "fastjango.auth.password_validation.NumericPasswordValidator",
    },
]

# Internationalization
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True

# Static files (CSS, JavaScript, Images)
STATIC_URL = "static/"
STATIC_ROOT = BASE_DIR / "staticfiles"
STATICFILES_DIRS = [BASE_DIR / "static"]

# Media files
MEDIA_URL = "media/"
MEDIA_ROOT = BASE_DIR / "media"

# Default primary key field type
DEFAULT_AUTO_FIELD = "fastjango.db.models.BigAutoField" 


#!/usr/bin/env python
"""
Tests for FastJango template rendering functionality.
"""

import os
import sys
import unittest
import tempfile
import shutil
from pathlib import Path

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango template components
from fastjango.templates import Template, get_template, render_to_string
from fastjango.template.loaders import FileSystemLoader, PackageLoader
from fastjango.template.context import Context


class TemplateTest(unittest.TestCase):
    """Test suite for template functionality."""
    
    def setUp(self):
        """Set up test environment with temporary template directory."""
        # Create a temporary directory for templates
        self.template_dir = tempfile.mkdtemp()
        
        # Create some test templates
        self.create_template("base.html", """
        <!DOCTYPE html>
        <html>
        <head>
            <title>{% block title %}Default Title{% endblock %}</title>
        </head>
        <body>
            <header>
                <h1>{% block header %}Header{% endblock %}</h1>
            </header>
            <main>
                {% block content %}{% endblock %}
            </main>
            <footer>
                {% block footer %}Footer{% endblock %}
            </footer>
        </body>
        </html>
        """)
        
        self.create_template("page.html", """
        {% extends "base.html" %}
        
        {% block title %}My Page{% endblock %}
        
        {% block header %}Welcome to My Page{% endblock %}
        
        {% block content %}
            <p>This is the content of my page.</p>
            <p>Hello, {{ name }}!</p>
            
            {% if show_list %}
                <ul>
                {% for item in items %}
                    <li>{{ item }}</li>
                {% endfor %}
                </ul>
            {% endif %}
        {% endblock %}
        """)
        
        self.create_template("simple.html", "Hello, {{ name }}!")
        
        # Initialize the template loader
        self.loader = FileSystemLoader(self.template_dir)
    
    def tearDown(self):
        """Clean up temporary files."""
        shutil.rmtree(self.template_dir)
    
    def create_template(self, name, content):
        """Create a template file in the temporary directory."""
        template_path = os.path.join(self.template_dir, name)
        with open(template_path, "w") as f:
            f.write(content)
    
    def test_simple_template(self):
        """Test rendering a simple template."""
        template = Template("Hello, {{ name }}!")
        context = {"name": "World"}
        rendered = template.render(context)
        self.assertEqual(rendered, "Hello, World!")
    
    def test_template_with_loop(self):
        """Test rendering a template with a loop."""
        template = Template("""
        <ul>
        {% for item in items %}
            <li>{{ item }}</li>
        {% endfor %}
        </ul>
        """)
        context = {"items": ["apple", "banana", "cherry"]}
        rendered = template.render(context)
        
        # Normalize whitespace for comparison
        rendered = " ".join(rendered.split())
        expected = "<ul> <li>apple</li> <li>banana</li> <li>cherry</li> </ul>"
        self.assertEqual(rendered, expected)
    
    def test_template_with_conditional(self):
        """Test rendering a template with conditionals."""
        template = Template("""
        {% if show_greeting %}
            <p>Hello, {{ name }}!</p>
        {% else %}
            <p>No greeting.</p>
        {% endif %}
        """)
        
        # Test with show_greeting=True
        context1 = {"show_greeting": True, "name": "World"}
        rendered1 = " ".join(template.render(context1).split())
        self.assertEqual(rendered1, "<p>Hello, World!</p>")
        
        # Test with show_greeting=False
        context2 = {"show_greeting": False, "name": "World"}
        rendered2 = " ".join(template.render(context2).split())
        self.assertEqual(rendered2, "<p>No greeting.</p>")
    
    def test_template_inheritance(self):
        """Test template inheritance."""
        # Load the template from the file system
        template = self.loader.get_template("page.html")
        
        # Render with context
        context = {
            "name": "John",
            "show_list": True,
            "items": ["Item 1", "Item 2", "Item 3"]
        }
        rendered = template.render(context)
        
        # Check that blocks from the parent template are present
        self.assertIn("<title>My Page</title>", rendered)
        self.assertIn("<h1>Welcome to My Page</h1>", rendered)
        
        # Check that the content block has been filled
        self.assertIn("<p>This is the content of my page.</p>", rendered)
        self.assertIn("<p>Hello, John!</p>", rendered)
        
        # Check that the loop rendered correctly
        self.assertIn("<li>Item 1</li>", rendered)
        self.assertIn("<li>Item 2</li>", rendered)
        self.assertIn("<li>Item 3</li>", rendered)
    
    def test_get_template(self):
        """Test the get_template function."""
        # Set up the template directory in settings
        import fastjango.conf
        fastjango.conf.settings.TEMPLATES = [{
            "BACKEND": "fastjango.template.backends.jinja2.Jinja2",
            "DIRS": [self.template_dir],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": []
            }
        }]
        
        # Get a template
        template = get_template("simple.html")
        rendered = template.render({"name": "World"})
        self.assertEqual(rendered, "Hello, World!")
    
    def test_render_to_string(self):
        """Test the render_to_string function."""
        # Set up the template directory in settings
        import fastjango.conf
        fastjango.conf.settings.TEMPLATES = [{
            "BACKEND": "fastjango.template.backends.jinja2.Jinja2",
            "DIRS": [self.template_dir],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": []
            }
        }]
        
        # Render a template to string
        rendered = render_to_string("simple.html", {"name": "World"})
        self.assertEqual(rendered, "Hello, World!")


class TemplateResponseTest(unittest.TestCase):
    """Test suite for TemplateResponse."""
    
    def setUp(self):
        """Set up test environment."""
        # Create a temporary directory for templates
        self.template_dir = tempfile.mkdtemp()
        
        # Create a test template
        self.create_template("response.html", """
        <!DOCTYPE html>
        <html>
        <head>
            <title>{{ title }}</title>
        </head>
        <body>
            <h1>{{ title }}</h1>
            <p>{{ message }}</p>
        </body>
        </html>
        """)
        
        # Set up the template directory in settings
        import fastjango.conf
        fastjango.conf.settings.TEMPLATES = [{
            "BACKEND": "fastjango.template.backends.jinja2.Jinja2",
            "DIRS": [self.template_dir],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": []
            }
        }]
    
    def tearDown(self):
        """Clean up temporary files."""
        shutil.rmtree(self.template_dir)
    
    def create_template(self, name, content):
        """Create a template file in the temporary directory."""
        template_path = os.path.join(self.template_dir, name)
        with open(template_path, "w") as f:
            f.write(content)
    
    def test_template_response(self):
        """Test TemplateResponse."""
        from fastjango.template.response import TemplateResponse
        
        # Create a TemplateResponse
        context = {"title": "Test Page", "message": "Hello, World!"}
        response = TemplateResponse("response.html", context)
        
        # Check the response
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, "response.html")
        self.assertEqual(response.context_data, context)
        
        # Render the response
        content = response.body.decode("utf-8")
        self.assertIn("<title>Test Page</title>", content)
        self.assertIn("<h1>Test Page</h1>", content)
        self.assertIn("<p>Hello, World!</p>", content)


def run_tests():
    """Run the template tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


"""
API routes for {{app_name}} app.
"""

from fastapi import APIRouter, HTTPException, Depends, status
from typing import List, Optional

from fastjango.core.dependencies import get_current_user
from .schemas import {{app_name_camel}}Create, {{app_name_camel}}Read, {{app_name_camel}}Update
from .services import {{app_name_camel}}Service

router = APIRouter(prefix="/{{app_name_snake}}", tags=["{{app_name}}"])
service = {{app_name_camel}}Service()


@router.get("/", response_model=List[{{app_name_camel}}Read])
async def list_{{app_name_snake}}s(
    skip: int = 0,
    limit: int = 100,
    current_user = Depends(get_current_user)
):
    """
    List all {{app_name}}s.
    """
    return await service.get_all(skip=skip, limit=limit)


@router.post("/", response_model={{app_name_camel}}Read, status_code=status.HTTP_201_CREATED)
async def create_{{app_name_snake}}(
    item: {{app_name_camel}}Create,
    current_user = Depends(get_current_user)
):
    """
    Create a new {{app_name}}.
    """
    return await service.create(item)


@router.get("/{item_id}", response_model={{app_name_camel}}Read)
async def read_{{app_name_snake}}(
    item_id: int,
    current_user = Depends(get_current_user)
):
    """
    Get a specific {{app_name}} by ID.
    """
    item = await service.get_by_id(item_id)
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="{{app_name_camel}} not found"
        )
    return item


@router.put("/{item_id}", response_model={{app_name_camel}}Read)
async def update_{{app_name_snake}}(
    item_id: int,
    item: {{app_name_camel}}Update,
    current_user = Depends(get_current_user)
):
    """
    Update a {{app_name}}.
    """
    updated_item = await service.update(item_id, item)
    if not updated_item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="{{app_name_camel}} not found"
        )
    return updated_item


@router.delete("/{item_id}", status_code=status.HTTP_204_NO_CONTENT)
async def delete_{{app_name_snake}}(
    item_id: int,
    current_user = Depends(get_current_user)
):
    """
    Delete a {{app_name}}.
    """
    success = await service.delete(item_id)
    if not success:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="{{app_name_camel}} not found"
        )
    return {} 


"""
Models for {{app_name}} app.
"""

from fastjango.db import models
from fastjango.core.exceptions import ValidationError


class {{app_name_camel}}Model(models.Model):
    """
    Example model for {{app_name}} app.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    
    class Meta:
        ordering = ["-created_at"]
        verbose_name = "{{app_name_camel}}"
        verbose_name_plural = "{{app_name_camel}}s"
    
    def __str__(self):
        return self.name
    
    def clean(self):
        """
        Custom validation logic.
        """
        if self.name.lower() == "test":
            raise ValidationError({"name": "Name cannot be 'test'"})
        
    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs) 


"""
Services for {{app_name}} app.
"""

from typing import List, Optional
import logging

from fastjango.core.exceptions import ServiceError
from .models import {{app_name_camel}}Model
from .schemas import {{app_name_camel}}Create, {{app_name_camel}}Update

logger = logging.getLogger(__name__)


class {{app_name_camel}}Service:
    """
    Service for handling {{app_name_camel}} operations.
    """
    
    async def get_all(self, skip: int = 0, limit: int = 100) -> List[{{app_name_camel}}Model]:
        """
        Get all {{app_name_camel}} objects.
        """
        try:
            return {{app_name_camel}}Model.objects.all()[skip:skip+limit]
        except Exception as e:
            logger.error(f"Error fetching {{app_name_camel}} list: {str(e)}", exc_info=True)
            raise ServiceError(f"Failed to fetch {{app_name_camel}} list")
    
    async def get_by_id(self, item_id: int) -> Optional[{{app_name_camel}}Model]:
        """
        Get a {{app_name_camel}} by ID.
        """
        try:
            return {{app_name_camel}}Model.objects.filter(id=item_id).first()
        except Exception as e:
            logger.error(f"Error fetching {{app_name_camel}} with ID {item_id}: {str(e)}", exc_info=True)
            raise ServiceError(f"Failed to fetch {{app_name_camel}}")
    
    async def create(self, item: {{app_name_camel}}Create) -> {{app_name_camel}}Model:
        """
        Create a new {{app_name_camel}}.
        """
        try:
            return {{app_name_camel}}Model.objects.create(**item.dict())
        except Exception as e:
            logger.error(f"Error creating {{app_name_camel}}: {str(e)}", exc_info=True)
            raise ServiceError(f"Failed to create {{app_name_camel}}")
    
    async def update(self, item_id: int, item: {{app_name_camel}}Update) -> Optional[{{app_name_camel}}Model]:
        """
        Update a {{app_name_camel}}.
        """
        try:
            obj = await self.get_by_id(item_id)
            if not obj:
                return None
            
            # Update fields that are present in the request
            for field, value in item.dict(exclude_unset=True).items():
                setattr(obj, field, value)
            
            obj.save()
            return obj
        except Exception as e:
            logger.error(f"Error updating {{app_name_camel}} with ID {item_id}: {str(e)}", exc_info=True)
            raise ServiceError(f"Failed to update {{app_name_camel}}")
    
    async def delete(self, item_id: int) -> bool:
        """
        Delete a {{app_name_camel}}.
        """
        try:
            obj = await self.get_by_id(item_id)
            if not obj:
                return False
            
            obj.delete()
            return True
        except Exception as e:
            logger.error(f"Error deleting {{app_name_camel}} with ID {item_id}: {str(e)}", exc_info=True)
            raise ServiceError(f"Failed to delete {{app_name_camel}}") 


#!/usr/bin/env python
"""
Tests for FastJango settings configuration.
"""

import os
import sys
import unittest
import tempfile
from pathlib import Path

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango settings components
from fastjango.conf import settings, LazySettings, Settings, ImproperlyConfigured


class SettingsTest(unittest.TestCase):
    """Test suite for settings configuration."""
    
    def setUp(self):
        """Set up test environment."""
        # Create a temporary directory for a settings module
        self.temp_dir = tempfile.mkdtemp()
        sys.path.insert(0, self.temp_dir)
        
        # Create a test settings module
        self.settings_module = os.path.join(self.temp_dir, "test_settings_module.py")
        with open(self.settings_module, "w") as f:
            f.write("""
# Test settings module
DEBUG = True
SECRET_KEY = 'test-secret-key'
ALLOWED_HOSTS = ['localhost', '127.0.0.1']
INSTALLED_APPS = [
    'app1',
    'app2',
]
DATABASE = {
    'default': {
        'ENGINE': 'sqlite',
        'NAME': 'db.sqlite3',
    }
}
MIDDLEWARE = []
TEMPLATES = [{
    'BACKEND': 'fastjango.template.backends.jinja2.Jinja2',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': [],
    }
}]
STATIC_URL = '/static/'
STATIC_ROOT = 'staticfiles'
""")
    
    def tearDown(self):
        """Clean up after tests."""
        # Remove the temporary directory and settings module
        sys.path.remove(self.temp_dir)
        os.unlink(self.settings_module)
        os.rmdir(self.temp_dir)
    
    def test_lazy_settings(self):
        """Test LazySettings functionality."""
        # Create LazySettings with our test module
        lazy_settings = LazySettings()
        lazy_settings.configure(
            settings_module="test_settings_module"
        )
        
        # Test that settings are loaded correctly
        self.assertTrue(lazy_settings.DEBUG)
        self.assertEqual(lazy_settings.SECRET_KEY, 'test-secret-key')
        self.assertEqual(lazy_settings.ALLOWED_HOSTS, ['localhost', '127.0.0.1'])
        self.assertEqual(lazy_settings.INSTALLED_APPS, ['app1', 'app2'])
        self.assertEqual(lazy_settings.DATABASE['default']['ENGINE'], 'sqlite')
    
    def test_direct_settings(self):
        """Test direct Settings functionality."""
        # Create Settings directly
        direct_settings = Settings("test_settings_module")
        
        # Test that settings are loaded correctly
        self.assertTrue(direct_settings.DEBUG)
        self.assertEqual(direct_settings.SECRET_KEY, 'test-secret-key')
        self.assertEqual(direct_settings.ALLOWED_HOSTS, ['localhost', '127.0.0.1'])
        self.assertEqual(direct_settings.INSTALLED_APPS, ['app1', 'app2'])
        self.assertEqual(direct_settings.DATABASE['default']['ENGINE'], 'sqlite')
    
    def test_settings_modification(self):
        """Test that settings can be modified."""
        # Create Settings
        settings_obj = Settings("test_settings_module")
        
        # Modify a setting
        settings_obj.DEBUG = False
        self.assertFalse(settings_obj.DEBUG)
        
        # Add a new setting
        settings_obj.NEW_SETTING = 'new-value'
        self.assertEqual(settings_obj.NEW_SETTING, 'new-value')
    
    def test_settings_as_dict(self):
        """Test settings as a dictionary."""
        # Create Settings
        settings_obj = Settings("test_settings_module")
        
        # Convert to dictionary
        settings_dict = settings_obj.as_dict()
        
        # Check dictionary values
        self.assertTrue(settings_dict['DEBUG'])
        self.assertEqual(settings_dict['SECRET_KEY'], 'test-secret-key')
        self.assertEqual(settings_dict['ALLOWED_HOSTS'], ['localhost', '127.0.0.1'])
    
    def test_missing_settings(self):
        """Test behavior with missing settings."""
        # Create Settings
        settings_obj = Settings("test_settings_module")
        
        # Try to access a non-existent setting
        with self.assertRaises(AttributeError):
            value = settings_obj.NON_EXISTENT_SETTING
    
    def test_required_settings(self):
        """Test required settings."""
        # Create a settings module missing SECRET_KEY
        no_secret_settings = os.path.join(self.temp_dir, "no_secret_settings.py")
        with open(no_secret_settings, "w") as f:
            f.write("""
# Missing SECRET_KEY
DEBUG = True
ALLOWED_HOSTS = ['localhost']
""")
        
        # Create Settings object that checks for required settings
        def create_settings():
            settings_obj = Settings("no_secret_settings")
            settings_obj.validate_settings(['SECRET_KEY'])
        
        # Check that validation raises ImproperlyConfigured
        with self.assertRaises(ImproperlyConfigured):
            create_settings()
    
    def test_default_settings(self):
        """Test default settings."""
        # Create minimal settings module
        minimal_settings = os.path.join(self.temp_dir, "minimal_settings.py")
        with open(minimal_settings, "w") as f:
            f.write("""
# Minimal settings
SECRET_KEY = 'minimal-key'
""")
        
        # Create Settings with defaults
        settings_obj = Settings("minimal_settings")
        
        # Set default values
        settings_obj.set_defaults({
            'DEBUG': False,
            'ALLOWED_HOSTS': ['localhost'],
            'INSTALLED_APPS': [],
        })
        
        # Check that defaults were applied
        self.assertFalse(settings_obj.DEBUG)
        self.assertEqual(settings_obj.ALLOWED_HOSTS, ['localhost'])
        self.assertEqual(settings_obj.INSTALLED_APPS, [])
        
        # Original setting should still be there
        self.assertEqual(settings_obj.SECRET_KEY, 'minimal-key')


class GlobalSettingsTest(unittest.TestCase):
    """Test suite for global settings instance."""
    
    def test_global_settings(self):
        """Test the global settings object."""
        # The global settings object should be a LazySettings instance
        self.assertIsInstance(settings, LazySettings)
        
        # Try to access a setting (will use FASTJANGO_SETTINGS_MODULE if set)
        try:
            value = settings.SECRET_KEY
            # If we get here, a setting was found
            self.assertTrue(True)
        except (AttributeError, ImproperlyConfigured):
            # This is acceptable if no settings module is configured
            pass


def run_tests():
    """Run the settings tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests()



"""
Tests for {{app_name}} routes.
"""

import pytest
from fastapi.testclient import TestClient
from fastjango.test import TestCase

from ..routes import router


class {{app_name_camel}}RoutesTests(TestCase):
    """
    Tests for {{app_name_camel}} routes.
    """
    
    def setUp(self):
        """
        Set up test client.
        """
        self.client = TestClient(router)
        
    def test_list_{{app_name_snake}}s(self):
        """
        Test listing {{app_name_camel}}s.
        """
        response = self.client.get("/")
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.json(), list)
        
    def test_create_{{app_name_snake}}(self):
        """
        Test creating a {{app_name_camel}}.
        """
        data = {
            "name": "Test {{app_name_camel}}",
            "description": "This is a test"
        }
        response = self.client.post("/", json=data)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.json()["name"], data["name"]) 


"""
API URLs for {{project_name}} project.
"""

from fastjango.urls import path
from fastjango.http import JsonResponse

def api_root(request):
    return JsonResponse({"message": "{{project_name}} API v1.0"})

urlpatterns = [
    path("/", api_root),
] 


"""
Tests for {{app_name}} models.
"""

import pytest
from django.test import TestCase
from fastjango.core.exceptions import ValidationError

from ..models import {{app_name_camel}}Model


class {{app_name_camel}}ModelTests(TestCase):
    """
    Tests for {{app_name_camel}}Model.
    """
    
    def test_create_{{app_name_snake}}(self):
        """
        Test creating a {{app_name_camel}}.
        """
        obj = {{app_name_camel}}Model.objects.create(
            name="Test {{app_name_camel}}",
            description="This is a test"
        )
        self.assertEqual(obj.name, "Test {{app_name_camel}}")
        self.assertEqual(obj.description, "This is a test")
        
    def test_validation(self):
        """
        Test validation logic.
        """
        obj = {{app_name_camel}}Model(name="test")
        with self.assertRaises(ValidationError):
            obj.save() 


"""
ASGI config for {{project_name}} project.
"""

import os
from fastjango.core.asgi import get_asgi_application

os.environ.setdefault("FASTJANGO_SETTINGS_MODULE", "{{project_name_snake}}.settings")

application = get_asgi_application() 


#!/usr/bin/env python
"""
Tests for FastJango URL configuration functionality.
"""

import os
import sys
import unittest
from unittest.mock import MagicMock, patch

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango URL components
from fastjango.urls import (
    path,
    include,
    Path,
    URLResolver,
)
from fastjango.http import JsonResponse
from fastapi import APIRouter


# Define some test views
def index_view(request):
    return JsonResponse({"message": "Index"})

def detail_view(request, id):
    return JsonResponse({"id": id})

def another_view(request):
    return JsonResponse({"message": "Another"})


class URLPatternTest(unittest.TestCase):
    """Test suite for URL pattern creation."""
    
    def test_path_function(self):
        """Test the path function."""
        # Create a path
        url_path = path("/test", index_view, name="test")
        
        # Check that the path has the correct attributes
        self.assertIsInstance(url_path, Path)
        self.assertEqual(url_path.path, "test")  # Leading slash should be removed
        self.assertEqual(url_path.view, index_view)
        self.assertEqual(url_path.name, "test")
    
    def test_path_with_parameters(self):
        """Test path with parameters."""
        # Create a path with parameters
        url_path = path("/items/<int:id>", detail_view, name="item-detail")
        
        # Check that the path has the correct attributes
        self.assertEqual(url_path.path, "items/<int:id>")
        self.assertEqual(url_path.view, detail_view)
        self.assertEqual(url_path.name, "item-detail")


class IncludeTest(unittest.TestCase):
    """Test suite for include function."""
    
    def setUp(self):
        """Set up test environment."""
        # Create a test module with urlpatterns
        self.module_name = "test_urls_module"
        self.module = type(sys)(self.module_name)
        sys.modules[self.module_name] = self.module
        
        # Define urlpatterns in the module
        self.module.urlpatterns = [
            path("/", index_view, name="index"),
            path("/items/<int:id>", detail_view, name="item-detail"),
        ]
        
        # Add app_name to the module
        self.module.app_name = "test_app"
    
    def tearDown(self):
        """Clean up after tests."""
        # Remove the test module
        del sys.modules[self.module_name]
    
    def test_include_function(self):
        """Test the include function."""
        # Include the test module
        included = include(self.module_name)
        
        # Check that include returns the correct tuple
        self.assertIsInstance(included, tuple)
        self.assertEqual(len(included), 3)
        
        # Check the components of the tuple
        patterns, namespace, prefix = included
        
        # Check patterns
        self.assertEqual(len(patterns), 2)
        self.assertEqual(patterns[0].name, "index")
        self.assertEqual(patterns[1].name, "item-detail")
        
        # Check namespace
        self.assertEqual(namespace, "test_app")
        
        # Check prefix
        self.assertEqual(prefix, "test_urls_module")
    
    def test_include_with_namespace(self):
        """Test include with custom namespace."""
        # Include the test module with a custom namespace
        included = include(self.module_name, namespace="custom")
        
        # Check that namespace is overridden
        patterns, namespace, prefix = included
        self.assertEqual(namespace, "custom")


class URLResolverTest(unittest.TestCase):
    """Test suite for URLResolver."""
    
    def test_register_patterns(self):
        """Test registering URL patterns."""
        # Create URL patterns
        urlpatterns = [
            path("/", index_view, name="index"),
            path("/items/<int:id>", detail_view, name="item-detail"),
            path("/another", another_view, name="another"),
        ]
        
        # Create a router and resolver
        router = APIRouter()
        resolver = URLResolver(router)
        
        # Register the patterns
        resolver.register(urlpatterns)
        
        # Check that the patterns were registered
        self.assertEqual(len(router.routes), 3)
        
        # Check the paths
        route_paths = [route.path for route in router.routes]
        self.assertIn("/", route_paths)
        self.assertIn("/items/{id}", route_paths)  # Parameters should be converted
        self.assertIn("/another", route_paths)
        
        # Check the names
        route_names = [getattr(route, "name", None) for route in router.routes]
        self.assertIn("index", route_names)
        self.assertIn("item-detail", route_names)
        self.assertIn("another", route_names)
    
    def test_register_included_patterns(self):
        """Test registering patterns with includes."""
        # Create a test module with urlpatterns
        module_name = "test_included_urls"
        module = type(sys)(module_name)
        sys.modules[module_name] = module
        
        # Define urlpatterns in the module
        module.urlpatterns = [
            path("/included", index_view, name="included-index"),
            path("/included/<int:id>", detail_view, name="included-detail"),
        ]
        
        # Create URL patterns with include
        urlpatterns = [
            path("/", index_view, name="index"),
            path("/sub", include(module_name)),
        ]
        
        # Create a router and resolver
        router = APIRouter()
        resolver = URLResolver(router)
        
        # Register the patterns
        resolver.register(urlpatterns)
        
        # Check that all patterns were registered
        self.assertEqual(len(router.routes), 3)
        
        # Check the paths
        route_paths = [route.path for route in router.routes]
        self.assertIn("/", route_paths)
        self.assertIn("/sub/included", route_paths)
        self.assertIn("/sub/included/{id}", route_paths)
        
        # Clean up
        del sys.modules[module_name]
    
    def test_path_normalization(self):
        """Test path normalization."""
        # Create URL patterns with various path formats
        urlpatterns = [
            path("no-leading-slash", index_view, name="no-leading"),
            path("/with-leading-slash", index_view, name="with-leading"),
            path("trailing-slash/", index_view, name="trailing"),
            path("//double//slashes//", index_view, name="double"),
        ]
        
        # Create a router and resolver
        router = APIRouter()
        resolver = URLResolver(router)
        
        # Register the patterns
        resolver.register(urlpatterns)
        
        # Check that the paths were normalized
        route_paths = [route.path for route in router.routes]
        self.assertIn("/no-leading-slash", route_paths)
        self.assertIn("/with-leading-slash", route_paths)
        self.assertIn("/trailing-slash", route_paths)
        self.assertIn("/double/slashes", route_paths)
    
    def test_path_parameters(self):
        """Test path parameter conversion."""
        # Create URL patterns with various parameter formats
        urlpatterns = [
            path("/int/<int:id>", detail_view, name="int-param"),
            path("/str/<str:name>", detail_view, name="str-param"),
            path("/slug/<slug:slug>", detail_view, name="slug-param"),
            path("/uuid/<uuid:uuid>", detail_view, name="uuid-param"),
            path("/path/<path:path>", detail_view, name="path-param"),
            path("/multi/<int:id>/<str:name>", detail_view, name="multi-param"),
        ]
        
        # Create a router and resolver
        router = APIRouter()
        resolver = URLResolver(router)
        
        # Register the patterns
        resolver.register(urlpatterns)
        
        # Check that the parameters were converted
        route_paths = [route.path for route in router.routes]
        self.assertIn("/int/{id}", route_paths)
        self.assertIn("/str/{name}", route_paths)
        self.assertIn("/slug/{slug}", route_paths)
        self.assertIn("/uuid/{uuid}", route_paths)
        self.assertIn("/path/{path}", route_paths)
        self.assertIn("/multi/{id}/{name}", route_paths)


def run_tests():
    """Run the URL configuration tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


#!/usr/bin/env python
"""
Tests for FastJango CLI commands and arguments.
"""

import os
import sys
import unittest
from unittest.mock import patch, MagicMock
from io import StringIO

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango CLI components
from fastjango import __version__
from typer.testing import CliRunner


class CLITest(unittest.TestCase):
    """Test suite for CLI functionality."""
    
    def setUp(self):
        """Set up test environment."""
        self.runner = CliRunner(mix_stderr=False)
    
    def test_version_flag(self):
        """Test --version flag displays the correct version and exits."""
        # Import locally to avoid module-level imports during patch setup
        from fastjango.cli.main import app
        
        # Run with --version flag
        with patch("typer.Exit") as mock_exit:
            result = self.runner.invoke(app, ["--version"])
            
            # Check that exit was called once
            self.assertEqual(mock_exit.call_count, 1)
            
            # Check version output in console.print call
            self.assertIn(f"FastJango v{__version__}", result.stdout)
    
    def test_help_command(self):
        """Test help command displays proper help information."""
        # Import locally to use freshly instantiated app in this test
        from fastjango.cli.main import app
        
        # Run with --help flag
        result = self.runner.invoke(app, ["--help"])
        
        # Check that help text is displayed
        self.assertEqual(result.exit_code, 0)
        self.assertIn("FastJango command-line utility for administrative tasks", result.stdout)
        
        # Check that commands are listed
        self.assertIn("startproject", result.stdout)
        self.assertIn("startapp", result.stdout)
        self.assertIn("runserver", result.stdout)
    
    def test_command_help(self):
        """Test command-specific help displays proper information."""
        # Import locally to use freshly instantiated app in this test
        from fastjango.cli.main import app
        
        # Test help for startproject command
        result = self.runner.invoke(app, ["startproject", "--help"])
        
        # Check that command help is displayed
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Creates a new FastJango project with the given name", result.stdout)
        self.assertIn("--directory", result.stdout)
        
        # Test help for startapp command
        result = self.runner.invoke(app, ["startapp", "--help"])
        
        # Check that command help is displayed
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Creates a new app in the current FastJango project", result.stdout)
        
        # Test help for runserver command
        result = self.runner.invoke(app, ["runserver", "--help"])
        
        # Check that command help is displayed
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Runs the FastJango development server", result.stdout)
        self.assertIn("--host", result.stdout)
        self.assertIn("--port", result.stdout)
    
    def test_verbose_flag(self):
        """Test --verbose flag enables debug logging."""
        # Import locally to avoid module-level imports during patch setup
        from fastjango.cli.main import app
        import logging
        
        # Patch setup_logging to verify DEBUG level is used
        with patch("fastjango.core.logging.setup_logging") as mock_setup_logging:
            # Also patch the logger.debug method to verify the message
            with patch("fastjango.cli.main.logger.debug") as mock_debug:
                result = self.runner.invoke(app, ["--verbose"])
                
                # Check that setup_logging was called with DEBUG level
                mock_setup_logging.assert_called_once_with(logging.DEBUG)
                
                # Verify that "Verbose logging enabled" debug message was logged
                mock_debug.assert_called_once_with("Verbose logging enabled")
                
                # Check command completed successfully
                self.assertEqual(result.exit_code, 0)


def run_tests():
    """Run the CLI tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


"""
WSGI config for {{project_name}} project.
"""

import os
from fastjango.core.wsgi import get_wsgi_application

os.environ.setdefault("FASTJANGO_SETTINGS_MODULE", "{{project_name_snake}}.settings")

application = get_wsgi_application() 


#!/usr/bin/env python
"""
Tests for FastJango form handling functionality.
"""

import os
import sys
import unittest
from decimal import Decimal
from datetime import date, datetime, time

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango form components
from fastjango.forms import (
    Form,
    CharField,
    BooleanField,
    IntegerField,
    DecimalField,
    EmailField,
    URLField,
    DateField,
    TimeField,
    DateTimeField,
    ChoiceField,
    MultipleChoiceField,
    ValidationError,
)


class ContactForm(Form):
    """Example contact form for testing."""
    
    name = CharField(max_length=100)
    email = EmailField()
    subject = CharField(max_length=200)
    message = CharField(max_length=1000, required=False)
    cc_myself = BooleanField(required=False)


class SignupForm(Form):
    """Example signup form with various fields for testing."""
    
    username = CharField(min_length=3, max_length=30)
    email = EmailField()
    password = CharField(min_length=8)
    confirm_password = CharField()
    age = IntegerField(min_value=18)
    newsletter = BooleanField(required=False)
    website = URLField(required=False)
    birthdate = DateField(required=False)
    preferred_time = TimeField(required=False)
    registration_date = DateTimeField(required=False)
    
    subscription = ChoiceField(choices=[
        ('free', 'Free'),
        ('basic', 'Basic'),
        ('premium', 'Premium'),
    ])
    
    interests = MultipleChoiceField(choices=[
        ('tech', 'Technology'),
        ('sports', 'Sports'),
        ('music', 'Music'),
        ('travel', 'Travel'),
    ], required=False)
    
    def clean(self):
        """Custom validation for the entire form."""
        cleaned_data = super().clean()
        
        # Check if passwords match
        password = cleaned_data.get("password")
        confirm_password = cleaned_data.get("confirm_password")
        
        if password and confirm_password and password != confirm_password:
            raise ValidationError("Passwords do not match")
        
        return cleaned_data


class SimpleFormTest(unittest.TestCase):
    """Test suite for simple form functionality."""
    
    def test_valid_form(self):
        """Test a valid form."""
        form_data = {
            "name": "John Doe",
            "email": "john@example.com",
            "subject": "Test Subject",
            "message": "This is a test message.",
            "cc_myself": True,
        }
        
        form = ContactForm(data=form_data)
        self.assertTrue(form.is_valid())
        
        cleaned_data = form.cleaned_data
        self.assertEqual(cleaned_data["name"], "John Doe")
        self.assertEqual(cleaned_data["email"], "john@example.com")
        self.assertEqual(cleaned_data["subject"], "Test Subject")
        self.assertEqual(cleaned_data["message"], "This is a test message.")
        self.assertTrue(cleaned_data["cc_myself"])
    
    def test_invalid_form(self):
        """Test an invalid form."""
        form_data = {
            "name": "John Doe",
            "email": "invalid-email",  # Invalid email
            "subject": "Test Subject",
        }
        
        form = ContactForm(data=form_data)
        self.assertFalse(form.is_valid())
        self.assertIn("email", form.errors)
    
    def test_required_fields(self):
        """Test required fields."""
        form_data = {
            "name": "John Doe",
            # Missing required email field
            # Missing required subject field
            "message": "This is a test message.",
        }
        
        form = ContactForm(data=form_data)
        self.assertFalse(form.is_valid())
        self.assertIn("email", form.errors)
        self.assertIn("subject", form.errors)
    
    def test_optional_fields(self):
        """Test optional fields."""
        form_data = {
            "name": "John Doe",
            "email": "john@example.com",
            "subject": "Test Subject",
            # Optional message field omitted
            # Optional cc_myself field omitted
        }
        
        form = ContactForm(data=form_data)
        self.assertTrue(form.is_valid())
        
        cleaned_data = form.cleaned_data
        self.assertEqual(cleaned_data.get("message", ""), "")
        self.assertFalse(cleaned_data.get("cc_myself", True))  # Default to False


class FieldValidationTest(unittest.TestCase):
    """Test suite for field validation."""
    
    def test_char_field(self):
        """Test CharField validation."""
        class TestForm(Form):
            name = CharField(min_length=2, max_length=10)
        
        # Valid
        form1 = TestForm(data={"name": "John"})
        self.assertTrue(form1.is_valid())
        
        # Too short
        form2 = TestForm(data={"name": "J"})
        self.assertFalse(form2.is_valid())
        
        # Too long
        form3 = TestForm(data={"name": "John Doe Smith"})
        self.assertFalse(form3.is_valid())
    
    def test_integer_field(self):
        """Test IntegerField validation."""
        class TestForm(Form):
            age = IntegerField(min_value=18, max_value=99)
        
        # Valid
        form1 = TestForm(data={"age": "25"})
        self.assertTrue(form1.is_valid())
        self.assertEqual(form1.cleaned_data["age"], 25)
        
        # Too small
        form2 = TestForm(data={"age": "16"})
        self.assertFalse(form2.is_valid())
        
        # Too large
        form3 = TestForm(data={"age": "100"})
        self.assertFalse(form3.is_valid())
        
        # Not an integer
        form4 = TestForm(data={"age": "twenty"})
        self.assertFalse(form4.is_valid())
    
    def test_decimal_field(self):
        """Test DecimalField validation."""
        class TestForm(Form):
            price = DecimalField(max_digits=5, decimal_places=2)
        
        # Valid
        form1 = TestForm(data={"price": "99.99"})
        self.assertTrue(form1.is_valid())
        self.assertEqual(form1.cleaned_data["price"], Decimal("99.99"))
        
        # Too many digits
        form2 = TestForm(data={"price": "1000.00"})
        self.assertFalse(form2.is_valid())
        
        # Too many decimal places
        form3 = TestForm(data={"price": "99.999"})
        self.assertFalse(form3.is_valid())
        
        # Not a number
        form4 = TestForm(data={"price": "price"})
        self.assertFalse(form4.is_valid())
    
    def test_email_field(self):
        """Test EmailField validation."""
        class TestForm(Form):
            email = EmailField()
        
        # Valid
        form1 = TestForm(data={"email": "test@example.com"})
        self.assertTrue(form1.is_valid())
        
        # Invalid
        form2 = TestForm(data={"email": "invalid-email"})
        self.assertFalse(form2.is_valid())
    
    def test_url_field(self):
        """Test URLField validation."""
        class TestForm(Form):
            website = URLField()
        
        # Valid
        form1 = TestForm(data={"website": "https://example.com"})
        self.assertTrue(form1.is_valid())
        
        # Invalid
        form2 = TestForm(data={"website": "invalid-url"})
        self.assertFalse(form2.is_valid())
    
    def test_date_field(self):
        """Test DateField validation."""
        class TestForm(Form):
            start_date = DateField()
        
        # Valid (YYYY-MM-DD)
        form1 = TestForm(data={"start_date": "2023-05-15"})
        self.assertTrue(form1.is_valid())
        self.assertEqual(form1.cleaned_data["start_date"], date(2023, 5, 15))
        
        # Invalid
        form2 = TestForm(data={"start_date": "not-a-date"})
        self.assertFalse(form2.is_valid())
    
    def test_time_field(self):
        """Test TimeField validation."""
        class TestForm(Form):
            start_time = TimeField()
        
        # Valid (HH:MM:SS)
        form1 = TestForm(data={"start_time": "14:30:00"})
        self.assertTrue(form1.is_valid())
        self.assertEqual(form1.cleaned_data["start_time"], time(14, 30, 0))
        
        # Also valid (HH:MM)
        form2 = TestForm(data={"start_time": "14:30"})
        self.assertTrue(form2.is_valid())
        self.assertEqual(form2.cleaned_data["start_time"], time(14, 30))
        
        # Invalid
        form3 = TestForm(data={"start_time": "not-a-time"})
        self.assertFalse(form3.is_valid())
    
    def test_choice_field(self):
        """Test ChoiceField validation."""
        class TestForm(Form):
            color = ChoiceField(choices=[
                ('red', 'Red'),
                ('green', 'Green'),
                ('blue', 'Blue'),
            ])
        
        # Valid
        form1 = TestForm(data={"color": "red"})
        self.assertTrue(form1.is_valid())
        
        # Invalid (not in choices)
        form2 = TestForm(data={"color": "purple"})
        self.assertFalse(form2.is_valid())
    
    def test_multiple_choice_field(self):
        """Test MultipleChoiceField validation."""
        class TestForm(Form):
            colors = MultipleChoiceField(choices=[
                ('red', 'Red'),
                ('green', 'Green'),
                ('blue', 'Blue'),
            ])
        
        # Valid (single choice)
        form1 = TestForm(data={"colors": ["red"]})
        self.assertTrue(form1.is_valid())
        
        # Valid (multiple choices)
        form2 = TestForm(data={"colors": ["red", "blue"]})
        self.assertTrue(form2.is_valid())
        self.assertEqual(form2.cleaned_data["colors"], ["red", "blue"])
        
        # Invalid (not in choices)
        form3 = TestForm(data={"colors": ["red", "purple"]})
        self.assertFalse(form3.is_valid())


class ComplexFormTest(unittest.TestCase):
    """Test suite for complex form functionality."""
    
    def test_valid_signup_form(self):
        """Test a valid signup form with various fields."""
        form_data = {
            "username": "johndoe",
            "email": "john@example.com",
            "password": "password123",
            "confirm_password": "password123",
            "age": "25",
            "newsletter": "true",
            "website": "https://example.com",
            "birthdate": "1990-01-01",
            "preferred_time": "14:30",
            "registration_date": "2023-05-15T14:30:00",
            "subscription": "basic",
            "interests": ["tech", "music"],
        }
        
        form = SignupForm(data=form_data)
        self.assertTrue(form.is_valid())
        
        cleaned_data = form.cleaned_data
        self.assertEqual(cleaned_data["username"], "johndoe")
        self.assertEqual(cleaned_data["email"], "john@example.com")
        self.assertEqual(cleaned_data["password"], "password123")
        self.assertEqual(cleaned_data["confirm_password"], "password123")
        self.assertEqual(cleaned_data["age"], 25)
        self.assertTrue(cleaned_data["newsletter"])
        self.assertEqual(cleaned_data["website"], "https://example.com")
        self.assertEqual(cleaned_data["birthdate"], date(1990, 1, 1))
        self.assertEqual(cleaned_data["preferred_time"], time(14, 30))
        self.assertEqual(
            cleaned_data["registration_date"],
            datetime(2023, 5, 15, 14, 30, 0)
        )
        self.assertEqual(cleaned_data["subscription"], "basic")
        self.assertEqual(cleaned_data["interests"], ["tech", "music"])
    
    def test_passwords_not_matching(self):
        """Test form with passwords that don't match."""
        form_data = {
            "username": "johndoe",
            "email": "john@example.com",
            "password": "password123",
            "confirm_password": "different",  # Different password
            "age": "25",
            "subscription": "basic",
        }
        
        form = SignupForm(data=form_data)
        self.assertFalse(form.is_valid())
        self.assertIn("__all__", form.errors)  # Form-level error


def run_tests():
    """Run the form tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


"""
URL routing utilities for FastJango.
"""

import re
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from fastapi import APIRouter, Depends, HTTPException, Request, Response
from fastapi.routing import APIRoute

from fastjango.core.dependencies import get_current_user
from fastjango.http import HttpResponse


class Path:
    """
    A path definition for URL routing.
    """
    
    def __init__(self, path: str, view: Callable, name: Optional[str] = None):
        """
        Initialize a path.
        
        Args:
            path: The URL path
            view: The view function or class
            name: The name of the path
        """
        # Ensure path doesn't start with a slash for consistency
        self.path = path[1:] if path.startswith('/') else path
        self.view = view
        self.name = name
        self.methods = ["GET"]  # Default to GET method
        self.kwargs = {}  # Additional kwargs for add_api_route


def path(path: str, view: Callable, name: Optional[str] = None) -> Path:
    """
    Define a path for URL routing.
    
    Args:
        path: The URL path
        view: The view function or class
        name: The name of the path
        
    Returns:
        A Path object
    """
    return Path(path, view, name)


class Include:
    """
    A class to store included URL patterns.
    """
    
    def __init__(self, patterns: List[Path], namespace: Optional[str], path: str):
        """
        Initialize an include.
        
        Args:
            patterns: The URL patterns to include
            namespace: The namespace for the included patterns
            path: The path prefix for the included patterns
        """
        self.patterns = patterns
        self.namespace = namespace
        self.path = path


def include(module_path: str, namespace: Optional[str] = None) -> Tuple[List[Path], Optional[str], str]:
    """
    Include paths from another module.
    
    Args:
        module_path: The module path to include
        namespace: Optional namespace override
        
    Returns:
        A tuple of (paths, namespace, module_path)
    """
    # Import the module
    import importlib
    
    try:
        module = importlib.import_module(module_path)
    except ImportError as e:
        raise ImportError(f"Could not import '{module_path}'. {e}")
    
    # Get the urlpatterns from the module
    urlpatterns = getattr(module, "urlpatterns", [])
    
    # Get the app_name from the module or use the provided namespace
    app_name = namespace or getattr(module, "app_name", None)
    
    # Extract the path prefix - use the last component of the module path 
    # that isn't 'urls' (which is a convention, not part of the URL)
    path_parts = module_path.split(".")
    path_prefix = ""
    
    for part in reversed(path_parts):
        if part != "urls":
            path_prefix = part
            break
    
    # Ensure path_prefix doesn't start with a slash
    if path_prefix.startswith('/'):
        path_prefix = path_prefix[1:]
    
    # Return a tuple with the urlpatterns, app_name and path_prefix
    return urlpatterns, app_name, path_prefix


class URLResolver:
    """
    A URL resolver for FastJango.
    """
    
    def __init__(self, router: APIRouter) -> None:
        """
        Initialize the URL resolver.
        
        Args:
            router: The FastAPI router to use
        """
        self.router = router

    def register(self, urlpatterns: List[Path], prefix: str = "") -> None:
        """
        Register URL patterns with the FastAPI router.
        
        Args:
            urlpatterns: The URL patterns to register
            prefix: The URL prefix to use
        """
        for path_obj in urlpatterns:
            if isinstance(path_obj.view, tuple):
                # This is a result of include()
                included_patterns, namespace, included_prefix = path_obj.view
                
                # Combine the prefixes properly
                combined_prefix = prefix
                if path_obj.path:
                    # Ensure path doesn't start with a slash when adding to prefix
                    route = path_obj.path
                    if route.startswith('/'):
                        route = route[1:]
                    
                    # Handle case where prefix is empty
                    if combined_prefix:
                        combined_prefix = f"{combined_prefix}/{route}"
                    else:
                        combined_prefix = route
                
                # Register included patterns with the combined prefix
                self.register(included_patterns, prefix=combined_prefix)
            else:
                # Convert the path to FastAPI format
                route_path = self._convert_path(path_obj.path)
                
                # Add prefix if present
                if prefix:
                    # Format prefix properly
                    clean_prefix = prefix
                    if clean_prefix.startswith('/'):
                        clean_prefix = clean_prefix[1:]
                    if clean_prefix.endswith('/'):
                        clean_prefix = clean_prefix[:-1]
                    
                    if route_path == "/":
                        route_path = f"/{clean_prefix}"
                    else:
                        route_path = f"/{clean_prefix}{route_path}"
                
                # Register the route with the FastAPI router
                try:
                    self.router.add_api_route(
                        route_path,
                        path_obj.view,
                        name=path_obj.name,
                    )
                except Exception as e:
                    from fastjango.core.logging import Logger
                    logger = Logger("fastjango.urls")
                    logger.error(f"Failed to register route {route_path}: {e}")
    
    def _convert_path(self, path: str) -> str:
        """
        Convert a Django-style path to a FastAPI path.
        
        Args:
            path: The Django-style path
            
        Returns:
            The FastAPI-style path
        """
        # Normalize multiple slashes to single slashes
        path = re.sub(r'//+', '/', path)
        
        # Replace Django-style path parameters with FastAPI-style
        # e.g., /<int:id>/ becomes /{id}/
        path = re.sub(r'<int:(\w+)>', r'{\1}', path)
        path = re.sub(r'<str:(\w+)>', r'{\1}', path)
        path = re.sub(r'<slug:(\w+)>', r'{\1}', path)
        path = re.sub(r'<uuid:(\w+)>', r'{\1}', path)
        path = re.sub(r'<path:(\w+)>', r'{\1}', path)
        
        # Remove trailing slash if present
        if path.endswith('/') and len(path) > 1:
            path = path[:-1]
        
        # Ensure path starts with a slash for FastAPI
        if not path.startswith('/'):
            path = '/' + path
        elif path == "":
            path = "/"
        
        return path 


#!/usr/bin/env python
"""
Test script for FastJango URL routing.
"""

import os
import sys
import shutil
import unittest
from fastapi import FastAPI, APIRouter

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# List of test fixtures to clean up
TEST_FIXTURES = [
    os.path.join(os.path.dirname(__file__), "test_app"),
    os.path.join(os.path.dirname(__file__), "nested_app"),
    os.path.join(os.path.dirname(__file__), "edge_cases"),
    os.path.join(os.path.dirname(__file__), "test_urls.py"),
    os.path.join(os.path.dirname(__file__), "test_settings.py")
]

def setup_test_fixtures():
    """Set up test fixtures needed for URL resolver testing."""
    # Setup FastJango environment
    os.environ["FASTJANGO_SETTINGS_MODULE"] = "tests.test_settings"

    # Create test settings
    test_settings_path = os.path.join(os.path.dirname(__file__), "test_settings.py")
    with open(test_settings_path, "w") as f:
        f.write("""
ROOT_URLCONF = "tests.test_urls"
DEBUG = True
INSTALLED_APPS = ["tests.test_app", "tests.nested_app", "tests.edge_cases"]
""")

    # Create test directories
    test_app_dir = os.path.join(os.path.dirname(__file__), "test_app")
    os.makedirs(test_app_dir, exist_ok=True)

    # Create test app's urls.py
    with open(os.path.join(test_app_dir, "urls.py"), "w") as f:
        f.write("""
from fastjango.urls import path, include
from fastjango.http import JsonResponse

def app_view(request):
    return JsonResponse({"message": "App view"})

def param_view(request, id):
    return JsonResponse({"id": id})

urlpatterns = [
    path("/", app_view, name="app-root"),
    path("/item/<int:id>", param_view, name="app-item"),
    path("/api", include("tests.test_app.api.urls")),
]

app_name = "test_app"
""")

    # Create test app's __init__.py
    with open(os.path.join(test_app_dir, "__init__.py"), "w") as f:
        f.write("")

    # Create test app's api directory
    api_dir = os.path.join(test_app_dir, "api")
    os.makedirs(api_dir, exist_ok=True)

    # Create test app's api/__init__.py
    with open(os.path.join(api_dir, "__init__.py"), "w") as f:
        f.write("")

    # Create test app's api/urls.py
    with open(os.path.join(api_dir, "urls.py"), "w") as f:
        f.write("""
from fastjango.urls import path, include
from fastjango.http import JsonResponse

def api_root(request):
    return JsonResponse({"message": "API root"})

def api_detail(request, item_id):
    return JsonResponse({"item_id": item_id})

urlpatterns = [
    path("/", api_root, name="api-root"),
    path("/<int:item_id>", api_detail, name="api-detail"),
    path("/v1", include("tests.test_app.api.v1.urls")),
]
""")

    # Create v1 API directory and files
    v1_dir = os.path.join(api_dir, "v1")
    os.makedirs(v1_dir, exist_ok=True)

    with open(os.path.join(v1_dir, "__init__.py"), "w") as f:
        f.write("")

    with open(os.path.join(v1_dir, "urls.py"), "w") as f:
        f.write("""
from fastjango.urls import path
from fastjango.http import JsonResponse

def v1_root(request):
    return JsonResponse({"message": "API v1"})

def v1_detail(request, id):
    return JsonResponse({"id": id, "version": "v1"})

urlpatterns = [
    path("/", v1_root, name="api-v1-root"),
    path("/<int:id>", v1_detail, name="api-v1-detail"),
]
""")

    # Create nested app
    nested_app_dir = os.path.join(os.path.dirname(__file__), "nested_app")
    os.makedirs(nested_app_dir, exist_ok=True)

    # Create nested app's __init__.py
    with open(os.path.join(nested_app_dir, "__init__.py"), "w") as f:
        f.write("")

    # Create nested app's urls.py
    with open(os.path.join(nested_app_dir, "urls.py"), "w") as f:
        f.write("""
from fastjango.urls import path
from fastjango.http import JsonResponse

def nested_view(request):
    return JsonResponse({"message": "Nested view"})

urlpatterns = [
    path("/", nested_view, name="nested-root"),
]

app_name = "nested"
""")

    # Create edge cases app
    edge_cases_dir = os.path.join(os.path.dirname(__file__), "edge_cases")
    os.makedirs(edge_cases_dir, exist_ok=True)

    with open(os.path.join(edge_cases_dir, "__init__.py"), "w") as f:
        f.write("")

    with open(os.path.join(edge_cases_dir, "urls.py"), "w") as f:
        f.write("""
from fastjango.urls import path
from fastjango.http import JsonResponse

def empty_path(request):
    return JsonResponse({"message": "Empty path"})

def trailing_slash(request):
    return JsonResponse({"message": "Trailing slash"})

def multiple_params(request, user_id, post_id):
    return JsonResponse({"user_id": user_id, "post_id": post_id})
    
def multiple_slashes(request):
    return JsonResponse({"message": "Multiple slashes"})

urlpatterns = [
    path("", empty_path, name="empty-path"),
    path("/trailing/", trailing_slash, name="trailing-slash"),
    path("/users/<int:user_id>/posts/<int:post_id>", multiple_params, name="multiple-params"),
    path("////multiple////slashes", multiple_slashes, name="multiple-slashes"),
]

app_name = "edge"
""")

    # Create test project urls.py
    test_urls_path = os.path.join(os.path.dirname(__file__), "test_urls.py")
    with open(test_urls_path, "w") as f:
        f.write("""
from fastjango.urls import path, include
from fastjango.http import JsonResponse

def index(request):
    return JsonResponse({"message": "Root view"})

def about(request):
    return JsonResponse({"message": "About view"})

urlpatterns = [
    path("/", index, name="index"),
    path("/about", about, name="about"),
    path("/app", include("tests.test_app.urls")),
    path("/nested", include("tests.nested_app.urls")),
    path("/edge", include("tests.edge_cases.urls")),
]
""")

    # Add __init__.py to tests directory
    tests_init_path = os.path.join(os.path.dirname(__file__), "__init__.py")
    if not os.path.exists(tests_init_path):
        with open(tests_init_path, "w") as f:
            f.write("")

def cleanup_test_fixtures():
    """Clean up test fixtures after tests have completed."""
    print("\nCleaning up test fixtures...")
    for fixture in TEST_FIXTURES:
        try:
            if os.path.isdir(fixture):
                shutil.rmtree(fixture)
                print(f"Removed directory: {os.path.basename(fixture)}")
            elif os.path.exists(fixture):
                os.remove(fixture)
                print(f"Removed file: {os.path.basename(fixture)}")
        except Exception as e:
            print(f"Error removing {fixture}: {e}")

# Import FastJango URL resolver
from fastjango.urls import URLResolver

def test_url_resolver():
    """Test the URL resolver."""
    print("Testing URL resolver...")
    
    # Import the URL patterns
    import importlib
    urls_module = importlib.import_module("tests.test_urls")
    urlpatterns = getattr(urls_module, "urlpatterns", [])
    
    # Create a router and resolver
    router = APIRouter()
    resolver = URLResolver(router)
    
    # Register the URL patterns
    resolver.register(urlpatterns)
    
    # Print registered routes
    print("\nRegistered routes:")
    for route in router.routes:
        print(f"Path: {route.path}, Name: {route.name}")

# Import and run FastJango ASGI app
def test_asgi_app():
    """Test the ASGI application."""
    print("\nTesting ASGI application...")
    
    # Import the ASGI application
    from fastjango.core.asgi import get_asgi_application
    
    # Create the ASGI application
    app = get_asgi_application()
    
    # Print registered routes from the FastAPI app
    print("\nRegistered ASGI routes:")
    for route in app.routes:
        if hasattr(route, "path"):
            print(f"Path: {route.path}, Name: {route.name}")

class TestURLResolver(unittest.TestCase):
    """Test case for URL resolver."""
    
    @classmethod
    def setUpClass(cls):
        """Set up test fixtures once for all tests."""
        setup_test_fixtures()
    
    @classmethod
    def tearDownClass(cls):
        """Clean up test fixtures after all tests have completed."""
        cleanup_test_fixtures()
    
    def test_resolver(self):
        """Test URL resolver functionality."""
        # Import the URL patterns
        import importlib
        urls_module = importlib.import_module("tests.test_urls")
        urlpatterns = getattr(urls_module, "urlpatterns", [])
        
        # Create a router and resolver
        router = APIRouter()
        resolver = URLResolver(router)
        
        # Register the URL patterns
        resolver.register(urlpatterns)
        
        # Check that routes are registered
        self.assertTrue(len(router.routes) > 0)
        
        # Check specific routes
        path_names = [route.name for route in router.routes if hasattr(route, 'name')]
        self.assertIn("index", path_names)
        self.assertIn("about", path_names)

if __name__ == "__main__":
    try:
        # For manual testing, setup fixtures and run tests directly
        setup_test_fixtures()
        test_url_resolver()
        #test_asgi_app()  # Uncomment to test ASGI app
    finally:
        # Always clean up, even if tests fail
        cleanup_test_fixtures() 



from fastjango.urls import path, include
from fastjango.http import JsonResponse

def index(request):
    return JsonResponse({"message": "Root view"})

def about(request):
    return JsonResponse({"message": "About view"})

urlpatterns = [
    path("/", index, name="index"),
    path("/about", about, name="about"),
    path("/app", include("tests.test_app.urls")),
    path("/nested", include("tests.nested_app.urls")),
    path("/edge", include("tests.edge_cases.urls")),
]



from setuptools import setup, find_packages
import os

# Read requirements
with open("requirements.txt") as f:
    requirements = [line.strip() for line in f if line.strip() and not line.startswith("#")]

# Read README for the long description
with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="fastjango",
    version="0.1.1",
    description="A fast web framework inspired by Django using FastAPI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Bhavik Shah",
    author_email="bhavik1st@gmail.com",
    url="https://github.com/fastjango/fastjango",
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "fastjango-admin=fastjango.cli.main:cli",
        ],
    },
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: FastAPI",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    keywords="fastapi, django, web framework, api, rest, fastjango",
    project_urls={
        "Documentation": "https://github.com/fastjango/fastjango",
        "Source": "https://github.com/fastjango/fastjango",
        "Tracker": "https://github.com/fastjango/fastjango/issues",
    },
) 


"""
Pydantic schemas for {{app_name}} app.
"""

from pydantic import BaseModel, Field
from typing import Optional
from datetime import datetime


class {{app_name_camel}}Base(BaseModel):
    """
    Base schema for {{app_name_camel}}.
    """
    name: str = Field(..., min_length=1, max_length=100)
    description: Optional[str] = Field(None, min_length=0, max_length=1000)


class {{app_name_camel}}Create({{app_name_camel}}Base):
    """
    Schema for creating a {{app_name_camel}}.
    """
    pass


class {{app_name_camel}}Update(BaseModel):
    """
    Schema for updating a {{app_name_camel}}.
    """
    name: Optional[str] = Field(None, min_length=1, max_length=100)
    description: Optional[str] = Field(None, min_length=0, max_length=1000)


class {{app_name_camel}}Read({{app_name_camel}}Base):
    """
    Schema for reading a {{app_name_camel}}.
    """
    id: int
    created_at: datetime
    updated_at: datetime
    
    class Config:
        orm_mode = True 


#!/usr/bin/env python
"""
Tests for FastJango HTTP response classes.
"""

import os
import sys
import json
import unittest
from typing import Dict, Any

# Add project root to path if script is run from tests directory
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import FastJango HTTP components
from fastjango.http import (
    HttpResponse,
    JsonResponse,
    FileResponse,
    RedirectResponse,
    TemplateResponse,
    StreamingResponse,
)


class HttpResponseTest(unittest.TestCase):
    """Test suite for HttpResponse class."""
    
    def test_basic_response(self):
        """Test basic HttpResponse functionality."""
        response = HttpResponse("Hello, world!")
        self.assertEqual(response.body, b"Hello, world!")
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content_type, "text/html")

    def test_custom_status_code(self):
        """Test HttpResponse with custom status code."""
        response = HttpResponse("Created", status_code=201)
        self.assertEqual(response.status_code, 201)
        
    def test_custom_headers(self):
        """Test HttpResponse with custom headers."""
        response = HttpResponse(
            content="Test",
            headers={"X-Custom-Header": "Value"}
        )
        self.assertEqual(response.headers.get("X-Custom-Header"), "Value")
        
    def test_content_type(self):
        """Test HttpResponse with custom content type."""
        response = HttpResponse("Text content", content_type="text/plain")
        self.assertEqual(response.content_type, "text/plain")


class JsonResponseTest(unittest.TestCase):
    """Test suite for JsonResponse class."""
    
    def test_basic_json_response(self):
        """Test basic JsonResponse functionality."""
        data = {"message": "Hello", "count": 42}
        response = JsonResponse(data)
        self.assertEqual(response.content_type, "application/json")
        
        # Parse the JSON body
        body_dict = json.loads(response.body.decode("utf-8"))
        self.assertEqual(body_dict, data)
        
    def test_json_with_custom_status(self):
        """Test JsonResponse with custom status code."""
        data = {"error": "Not found"}
        response = JsonResponse(data, status_code=404)
        self.assertEqual(response.status_code, 404)
        
    def test_json_with_non_dict(self):
        """Test JsonResponse with non-dict data."""
        data = [1, 2, 3, 4]
        response = JsonResponse(data)
        body_list = json.loads(response.body.decode("utf-8"))
        self.assertEqual(body_list, data)


class RedirectResponseTest(unittest.TestCase):
    """Test suite for RedirectResponse class."""
    
    def test_temporary_redirect(self):
        """Test temporary (302) redirect."""
        response = RedirectResponse("/new-location")
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.headers.get("location"), "/new-location")
        
    def test_permanent_redirect(self):
        """Test permanent (301) redirect."""
        response = RedirectResponse("/permanent", status_code=301)
        self.assertEqual(response.status_code, 301)
        self.assertEqual(response.headers.get("location"), "/permanent")


class StreamingResponseTest(unittest.TestCase):
    """Test suite for StreamingResponse class."""
    
    def test_streaming_response(self):
        """Test streaming response functionality."""
        def generator():
            for i in range(3):
                yield f"chunk {i}"
                
        response = StreamingResponse(generator())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content_type, "text/html")


def run_tests():
    """Run the HTTP response tests."""
    unittest.main(argv=['first-arg-is-ignored'], exit=False)


if __name__ == "__main__":
    run_tests() 


"""
HTTP utilities for FastJango.
"""

import json
from typing import Any, Dict, List, Optional, Union

from fastapi import Response
from fastapi.responses import JSONResponse, HTMLResponse, RedirectResponse
from fastapi.encoders import jsonable_encoder


class JsonResponse(JSONResponse):
    """
    A response that renders its content as JSON.
    """
    
    def __init__(
        self,
        content: Any,
        status_code: int = 200,
        headers: Optional[Dict[str, str]] = None,
        media_type: Optional[str] = None,
        background=None,
    ):
        """
        Initialize the JSON response.
        
        Args:
            content: The content to render
            status_code: The HTTP status code
            headers: HTTP headers
            media_type: The media type
            background: Background task
        """
        # Ensure content is JSON serializable
        content = jsonable_encoder(content)
        
        super().__init__(
            content=content,
            status_code=status_code,
            headers=headers,
            media_type=media_type,
            background=background,
        )


class HttpResponse(Response):
    """
    A basic HTTP response.
    """
    
    def __init__(
        self,
        content: Any = None,
        status_code: int = 200,
        headers: Optional[Dict[str, str]] = None,
        media_type: Optional[str] = None,
        background=None,
    ):
        """
        Initialize the HTTP response.
        
        Args:
            content: The content to render
            status_code: The HTTP status code
            headers: HTTP headers
            media_type: The media type
            background: Background task
        """
        if isinstance(content, str):
            pass
        elif content is not None:
            content = str(content)
        else:
            content = ""
        
        super().__init__(
            content=content,
            status_code=status_code,
            headers=headers,
            media_type=media_type or "text/html",
            background=background,
        )


class TemplateResponse(HTMLResponse):
    """
    A response that renders a template.
    """
    
    def __init__(
        self,
        request: Any,
        template_name: str,
        context: Optional[Dict[str, Any]] = None,
        status_code: int = 200,
        headers: Optional[Dict[str, str]] = None,
        media_type: Optional[str] = None,
        background=None,
    ):
        """
        Initialize the template response.
        
        Args:
            request: The request
            template_name: The name of the template to render
            context: The context to render the template with
            status_code: The HTTP status code
            headers: HTTP headers
            media_type: The media type
            background: Background task
        """
        from fastjango.template import render_to_string
        
        # Render template to string
        content = render_to_string(template_name, context or {}, request)
        
        super().__init__(
            content=content,
            status_code=status_code,
            headers=headers,
            media_type=media_type,
            background=background,
        )


def redirect(url: str, permanent: bool = False) -> RedirectResponse:
    """
    Return a redirect response.
    
    Args:
        url: The URL to redirect to
        permanent: Whether the redirect is permanent
        
    Returns:
        A redirect response
    """
    status_code = 301 if permanent else 302
    return RedirectResponse(url=url, status_code=status_code) 


"""
URL configuration for {{project_name}} project.
"""

from fastjango.urls import path, include
from fastjango.http import JsonResponse

# Define API endpoints
def index(request):
    return JsonResponse({"message": "Welcome to {{project_name}}"})

urlpatterns = [
    path("/", index),
    path("/api", include("{{project_name_snake}}.api.urls")),
] 